#
# This script gets the verbs in the slot "_ the living daylights"
# in a sort of roundabout way, which is more efficient than using
# the way implemented in "daylights_simple_conventional.r".
#
# This shows how you can work around the problem that CQP always
# starts by matching the leftmost token, which leads to very long
# processing time when there are many matches for the first token,
# as is the case with [pos='VB.*'], for example.
#
# It queries just the lexical material (fast!), then extends the
# matching region to the left, checks whether we have a verb there,
# then runs a frequency analysis of the actual verbs.
#
# Lines beginning with # are ignored by R (= comments).
#
# Self-expalnatory lines are not commented.
#
#

# Erase the entire workspace.
rm(list=ls(all=T))

# Make sure rcqp is loaded.
require(rcqp)

# A function that provides "press return to continue".
wait <- function(nextthing="")
{
  cat ("\n\nPress [ENTER] to see: ", nextthing)
  line <- readline()
}

# Set a character variable containing the (simple) query.
the.query="[word='living'] [word='daylights']"

# cat() just prints its arguments, "\n" is the line feed.
cat("\nPlease wait. Running CQP query: ", the.query)

# Save the current time in variable ptm.
ptm <- proc.time()

# This eexcutes the query and svaes it internally as "Daylights".
cqi_query("UKCOW2012D", "Daylights", the.query)

# Calculate teh time taken for the query.
ptm <- proc.time() - ptm
cat("\nTime taken:", ptm[3], "\n\n")

wait("concordance size")
print(cqi_subcorpus_size("UKCOW2012D:Daylights"))

wait("10 lines of the CQP dump")
print(cqi_dump_subcorpus("UKCOW2012D:Daylights", last=10))

# We dump the match and matchend positions in matrix dld. 
dld <- cqi_dump_subcorpus("UKCOW2012D:Daylights")

# Subtract two from the left column (=match) in dld.
# This effectively extends the match region by 2 positions to the left.
dld[,1] <- dld[,1]-2

wait("the lemmas of all matches at cpos-2")
print(cqi_cpos2str("UKCOW2012D.lemma", dld[,1]))

wait("the POS of all matches at cpos-2")
print(cqi_cpos2str("UKCOW2012D.pos", dld[,1]))

# Filter those (extended) matches which do not begin with a verb.
dlwhichvb <- grep("VB.*", cqi_cpos2str("UKCOW2012D.pos", dld[,1]))

# Extract the positions of the verbs from the match matrix. 
dlvb <- dld[dlwhichvb, 1]

# Get the verb lemmas, sort them, and count them.
# rle is a run length encoding function like Unix uniq -c.
dlc <- rle(sort(cqi_cpos2str("UKCOW2012D.lemma", dlvb)))

wait("lemmas of verbs at cpos-2 and their frequencies (alphabetically sorted)")
print(dlc)

wait("a bar plot of the above")
barplot(dlc$lengths, names.arg=dlc$values)

wait("the sort order to sort the above by frequency")
print(order(dlc$lengths,decreasing=T))

# Create a list similar to that returned in a rle object.
# Use the sort order to sort them by decreasing frequency.
dlco <- list()
dlco$lengths <- dlc$lengths[order(dlc$lengths,decreasing=T)]
dlco$values <- dlc$values[order(dlc$lengths,decreasing=T)]

wait("a bar plot of the above sorted by frequency")
barplot(dlco$lengths, names.arg=dlco$values, las=2)

cat("\nDone...\n")
